require("love_reactor/all")

Mario = class({
  sprite = love.graphics.newImage("sprites/hero.psd"),
  width = 64,
  height = 64,
  acceleration = {
    x = 0, y = 1000
  },
  animations = {
    stand = {
      frames = {2},
      speed = .25,
    },
    run = {
      frames = {1,2},
      speed = .25
    },
    jump = {
      frames = {3},
      speed = .25
    },
    fall = {
      frames = {4},
      speed = .25
    }
  }
}, {drawable, animatable, collidable, movable, animatable})

keypressed_stream
  .filter(function(k) return k == " " end)
  .map(function()
    for _, mario in pairs(Mario.instances.values()) do
      mario.velocity.y = -500
    end
  end)

keyheld_stream
  .map(function(k)
    local speed = ({
      ["left"] = {x = -1, y = 0},
      ["right"] = {x = 1, y = 0},
    })[k]

    if speed then
      for _, mario in pairs(Mario.instances.values()) do
        mario.velocity.x = speed.x * 100
      end
    end
  end)

keyreleased_stream
  .map(function(k)
    if ({["left"] = true, ["right"] = true})[k] then
      for _, mario in pairs(Mario.instances.values()) do
        mario.velocity.x = 0
      end
    end
  end)

update_stream
  .map(function()
    for _, mario in pairs(Mario.instances.values()) do
      if math.abs(mario.velocity.x) > 0 then
        mario:play("run")
      else
        mario:play("stand")
      end
    end
  end)

Goomber = class({
  sprite = love.graphics.newImage("sprites/goomber.psd"),
  width = 64,
  height = 64,
  animations = {
    stand = {
      frames = {1},
    },
    die = {
      frames = {2}
    }
  }
}, {drawable, movable, animatable, collidable})

Floor = class({
  x = 0,
  y = love.window.getHeight() - 64,
  width = love.window.getWidth(),
  height = 64,
  immobile = true
}, {collidable})

local font = love.graphics.newFont(30)
StompCounter = class({
  text = "Stomps: 1",
  stomps = 0,
  draw = function(self)
    love.graphics.setFont(font)
    love.graphics.print("Stomps: " .. self.stomps, love.window.getWidth() / 2 - 150, love.window.getHeight() / 2)
  end
}, {drawable, singleton})

collision_stream
  .map_filter(collision_between(Floor.instances, union(Goomber.instances, Mario.instances)))
  .map(function(e)
    simple_collisions(e.a, e.b)
    if e.b.velocity.y > 0 then
      e.b.velocity.y = 0
    end
  end)

collision_stream
  .filter(function(e) return e.type == "start" end)
  .map_filter(collision_between(Mario.instances, Goomber.instances))
  .map(function(e)
    if e.a.y < e.b.y then
      e.a.velocity.y = e.a.velocity.y * -1
      StompCounter.instance.stomps = StompCounter.instance.stomps + 1
    end
  end)

buy_mario = class({
  sprite = love.graphics.newImage("sprites/buy_mario.psd")
}, {drawable, clickable}).new({
  x = 64,
  y = 64
})
buy_mario.click_stream
  .filter(function(e) return e.type == "down" end)
  .map(function()
    if StompCounter.instance.stomps >= 10 then
      StompCounter.instance.stomps = StompCounter.instance.stomps - 10
      Mario.new({
        x = love.window.getWidth() / 2,
        y = love.window.getHeight() / 2
      })
    end
  end)

buy_goomber = class({
  sprite = love.graphics.newImage("sprites/buy_goomba.psd")
}, {drawable, clickable}).new({
  x = love.window.getWidth() - 64,
  y = 64
})
buy_goomber.click_stream
  .filter(function(e) return e.type == "down" end)
  .map(function()
    if StompCounter.instance.stomps >= 50 then
      StompCounter.instance.stomps = StompCounter.instance.stomps - 50
      Goomber.new({
        x = math.random(64, love.window.getWidth() - 64),
        y = love.window.getHeight() / 2
      })
    end
  end)

update_stream
  .map(function()
    buy_goomber.visible = StompCounter.instance.stomps >= 50
    buy_mario.visible = StompCounter.instance.stomps >= 10
  end)

function love.load()
  Mario.new({
    x = 64,
    y = love.window.getHeight() - 64
  })
  Goomber.new({
    x = love.window.getWidth() / 2,
    y = love.window.getHeight() - 64
  })

  Floor.new()

  StompCounter.new()
end
